@@ -152,13 +152,14 @@ tiledb_array_is_heterogeneous <- function(arr) {
 ##' @param arr A TileDB Array object as for example returned by \code{tiledb_array()}
 ##' @param ts_start A Datetime object that will be converted to millisecond granularity
 ##' @param ts_end A Datetime object that will be converted to millisecond granularity
+##' @param ctx A tiledb_ctx object (optional)
 ##' @return A boolean indicating success
 ##' @export
-tiledb_array_delete_fragments <- function(arr, ts_start, ts_end) {
+tiledb_array_delete_fragments <- function(arr, ts_start, ts_end, ctx = tiledb_get_context()) {
     stopifnot("The 'arr' argument must be a tiledb_array object" = .isArray(arr),
               "The 'ts_start' argument must a time object" = inherits(ts_start, "POSIXct"),
               "The 'ts_end' argument must a time object" = inherits(ts_end, "POSIXct"))
-    libtiledb_array_delete_fragments(arr@ptr, ts_start, ts_end)
+    libtiledb_array_delete_fragments(ctx@ptr, arr@ptr, ts_start, ts_end)
     invisible(TRUE)
 }
 
@@ -176,3 +177,33 @@ tiledb_array_has_enumeration <- function(arr) {
     }
     return(libtiledb_array_has_enumeration_vector(ctx@ptr, arr@ptr))
 }
+
+##' Run an aggregate query on the given array and attribute
+##'
+##' @param array A TileDB Array object
+##' @param attrname The name of an attribute
+##' @param operation The name of aggregation operation
+##' @param nullable A boolean toggle whether the attribute is nullable
+##' @return The value of the aggregation
+##' @export
+tiledb_array_apply_aggregate <- function(array, attrname,
+                                         operation = c("Count", "NullCount", "Min", "Max",
+                                                       "Mean", "Sum"),
+                                         nullable = TRUE) {
+    stopifnot("The 'query' argument must be a TileDB Array object" = is(array, "tiledb_array"),
+              "The 'attrname' argument must be character" = is.character(attrname),
+              "The 'operation' argument must be character" = is.character(operation),
+              "The 'nullable' argument must be logical" = is.logical(nullable))
+
+    operation <- match.arg(operation)
+
+    if (tiledb_array_is_open(array))
+        array <- tiledb_array_close(array)
+
+    query <- tiledb_query(array, "READ")
+
+    if (! tiledb_query_get_layout(query) %in% c("UNORDERED", "GLOBAL_ORDER"))
+        query <- tiledb_query_set_layout(query, "UNORDERED")
+
+    libtiledb_query_apply_aggregate(query@ptr, attrname, operation, nullable)
+}
