@@ -19,6 +19,7 @@ isrelease=$?
 echo "::group::Setup sources"
 if [ ${isrelease} -eq 0 ]; then
     git clone --single-branch --branch ${version} https://github.com/TileDB-Inc/TileDB.git TileDB-${version}
+    git log --graph --pretty=format:'%h - %d %s (%cr) <%an>' --abbrev-commit | head
 elif [ ${version} = "dev" ]; then
     wget https://github.com/TileDB-Inc/TileDB/archive/refs/heads/${version}.zip
     unzip ${version}.zip
@@ -38,7 +39,7 @@ echo "::group::Build from source"
 mkdir build
 cd build
 export AWSSDK_ROOT_DIR=/usr
-../bootstrap --prefix=/usr/local --enable-s3 --enable-serialization
+../bootstrap --prefix=/usr/local --enable-s3 --enable-serialization --linkage=shared
 make -j 8
 make -C tiledb install
 ldconfig
